uniform float4x4 World;
uniform float4x4 View;
uniform float4x4 Projection;
uniform float TweenValue = 0;
uniform float4 diffuseLightColor = float4(0,1,1,1);
uniform float3 lightPosition = float3(1,0,0);

texture2D Texture;
sampler2D ModelTextureSampler = sampler_state{Texture = <Texture>;MinFilter = Linear;MagFilter = Linear;MipFilter = Linear;AddressU = wrap;AddressV = wrap;};

struct VertexShaderInput
{
    float2 TextureCoordinate : TEXCOORD0;
    float4 Position1 : POSITION0;    
    float4 Position2 : POSITION1;
    float4 Normal1 : NORMAL0;
    float4 Normal2 : NORMAL1;

};

struct VertexShaderOutput
{
    float4 Position : POSITION0;
    float4 Color : COLOR0;
    float2 TextureCoordinate : TEXCOORD0;
};

VertexShaderOutput VertexShaderFunction(VertexShaderInput input)
{
    VertexShaderOutput output;
	
	float4 NewPosition = lerp(input.Position1, input.Position2, TweenValue);
	//NewPosition.y = 0;
	//NewPosition.xz = input.TextureCoordinate * 50;
	float4 NewNormal = lerp(input.Normal1, input.Normal2, TweenValue);
	
    float4 worldPosition = mul(NewPosition, World);
    float4 viewPosition = mul(worldPosition, View);
    output.Position = mul(viewPosition, Projection);
    
    ////get world space vectors
    float3 worldNormal =  mul(NewNormal, World);
    float4 worldPosition2 = mul(float4(NewPosition.xyz,1), World);
    worldPosition2 = worldPosition2 / worldPosition2.w;   
    
    float3 directionToLight = normalize(lightPosition);
    float diffuseIntensity = saturate( dot(directionToLight, worldNormal));
    float4 diffuse = diffuseLightColor * diffuseIntensity;

	output.TextureCoordinate = input.TextureCoordinate;
	output.Color = 0.5 + saturate(dot(normalize(worldPosition-lightPosition),normalize(NewNormal)));
	

    return output;
}

float4 PixelShaderFunction(VertexShaderOutput input) : COLOR0
{
	
	return tex2D(ModelTextureSampler, input.TextureCoordinate) * input.Color;
}

technique Technique1
{
    pass Pass1
    {
        VertexShader = compile vs_1_1 VertexShaderFunction();
        PixelShader = compile ps_1_1 PixelShaderFunction();
    }
}
