using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Media;
using Microsoft.Xna.Framework.Net;
using Microsoft.Xna.Framework.Storage;

namespace MorphTargetAnimation
{
    /// <summary>
    /// This is the main type for your game
    /// </summary>
    public class MorphTargetDemonstrationGame : Microsoft.Xna.Framework.Game
    {
        GraphicsDeviceManager graphics;
        MorphGameObject morphGameObject;

        // Sprite batch and sprite font to allow drawing text over the scene
        SpriteBatch spriteBatch;
        SpriteFont kootenaySize14SpriteFont;

        int linesDrawnThisFrame = 0;

        public MorphTargetDemonstrationGame()
        {
            graphics = new GraphicsDeviceManager(this);
            Content.RootDirectory = "Content";

            morphGameObject = new MorphGameObject(this, Content);
            Components.Add(morphGameObject);
        }

        /// <summary>
        /// Allows the game to perform any initialization it needs to before starting to run.
        /// This is where it can query for any required services and load any non-graphic
        /// related content.  Calling base.Initialize will enumerate through any components
        /// and initialize them as well.
        /// </summary>
        protected override void Initialize()
        {
            base.Initialize();
        }

        /// <summary>
        /// LoadContent will be called once per game and is the place to load
        /// all of your content.
        /// </summary>
        protected override void LoadContent()
        {
            // Create a new SpriteBatch, which can be used to draw textures.
            spriteBatch = new SpriteBatch(GraphicsDevice);
            kootenaySize14SpriteFont = Content.Load<SpriteFont>("KootenaySize14");
        }

        /// <summary>
        /// UnloadContent will be called once per game and is the place to unload
        /// all content.
        /// </summary>
        protected override void UnloadContent()
        {
        }

        /// <summary>
        /// Allows the game to run logic such as updating the world,
        /// checking for collisions, gathering input, and playing audio.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Update(GameTime gameTime)
        {
            // Allows the game to exit
            if (GamePad.GetState(PlayerIndex.One).Buttons.Back == ButtonState.Pressed
                || Keyboard.GetState().IsKeyDown(Keys.Escape))
                this.Exit();

            base.Update(gameTime);
        }

        /// <summary>
        /// This is called when the game should draw itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Draw(GameTime gameTime)
        {
            // Clear the back buffer each frame to sky blue - cornflower blue is so 2010
            GraphicsDevice.Clear(Color.SkyBlue);

            // Draw game components first
            base.Draw(gameTime);

            DrawTextOverlay();
        }

        private void DrawTextOverlay()
        {
            // Reset the number of text lines drawn
            linesDrawnThisFrame = 0;

            spriteBatch.Begin(SpriteBlendMode.AlphaBlend, SpriteSortMode.Deferred, SaveStateMode.SaveState);
            DrawTextLine("Morph Target Demonstration by Barnaby Smith");
            DrawTextLine("");
            DrawTextLine("Controls: WASD - rotate camera, ZX zoom camera, Q toggle wireframe");

            // If a 360 gamepad is detected notify user of additional controls
            if (GamePad.GetState(PlayerIndex.One).IsConnected)
            {
                DrawTextLine("Pad Controls: Right Thumbstick - rotate camera");
                DrawTextLine("Shoulder buttons - zoom camera, A toggle wireframe");
            }

            DrawTextLine("Tween Value: " + morphGameObject.Tween.ToString());
            DrawTextLine("Primary Model Index: " + morphGameObject.PrimaryMeshIndex.ToString());            
            spriteBatch.End();
        }
        private void DrawTextLine(string message)
        {
            Vector2 position = new Vector2(20, 20 + 20 * linesDrawnThisFrame);
            // Draw offset shadow
            spriteBatch.DrawString(kootenaySize14SpriteFont, message, position + new Vector2(1,1), Color.Black);
            // Draw main text
            spriteBatch.DrawString(kootenaySize14SpriteFont, message, position, Color.White);
            // Increment the line counter, so that the next call of this function will not overlay this text
            linesDrawnThisFrame++;
        }
    }
}
